import argparse
from yatrash.trash import trash, default_trash_dir


description = '''
description:
  Safely move files into a trash can instead of rm.

  - Files are stored by their path relative to either ~ or /
    (preferring ~ if possible).
  - If an existing item with the same name is already in the trash, I
    add _1, _2, ... to the end of the filename.
  - Metadata is copied; See Python documentation on shutil.copy2 for
    details.'''[1:]

example = '''
example usage:
  $ touch a
  $ mkdir b
  $ touch b/c
  $ touch /tmp/d
  $ trash a b/c /tmp/d
  /home/sam/subdir/a.txt -> /home/sam/.trash/subdir/a.txt
  /home/sam/subdir/b/c.txt -> /home/sam/.trash/subdir/b/c.txt
  /tmp/d.txt -> /home/sam/.trash/_root/tmp/d.txt
  $ touch a
  $ trash a
  /home/sam/subdir/a.txt -> /home/sam/.trash/subdir/a_1.txt
'''[1:]

trash_dir_help = '''
destination for trash (defaults to ${{TRASH_DIR}}, and then '{}')
'''.strip().format(default_trash_dir)

trash_files_help = '''
the files to remove
'''.strip()


def main():
    parser = argparse.ArgumentParser(
        description=description, epilog=example,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument('trash_files', type=str, nargs='+',
                        help=trash_files_help)
    parser.add_argument('--trash-dir', metavar='DIR', default='',
                        help=trash_dir_help)
    args = parser.parse_args()

    trash(args.trash_files, args.trash_dir)

    return 0


__all__ = ['main']



#!/usr/bin/env python3

import pathlib
from setuptools import setup, find_packages

description = 'Safely move files into a trash can instead of `rm`.'

classifiers='''
Natural Language :: English
Environment :: Console
License :: OSI Approved :: GNU General Public License v3 (GPLv3)
Operating System :: OS Independent
Programming Language :: Python :: 3.6
Topic :: Utilities
'''

setup(
    name='yatrash',
    version='0.1.3',
    description=description,
    author='Samuel Grayson',
    author_email='sam@samgrayson.me',
    url='https://github.com/charmoniumQ/yatrash/',
    packages=find_packages(),
    long_description=pathlib.Path("README.md").read_text(),
    long_description_content_type="text/markdown",
    python_requires='>=3.6.0',
    entry_points = {
        "console_scripts": [
            'trash = yatrash.__main__:main',
        ],
    },
    classifiers=[classifier.strip()
                 for classifier in classifiers.strip().split('\n')],
)



import os
import sys
import shutil
import itertools
from pathlib import Path


default_trash_dir = '~/.trash'


def candidate_names(path):
    '''unused_names('f.txt') yields f.txt, f_1.txt, f_2.txt, ...'''
    yield path
    for i in itertools.count(1):
        yield path.with_name('{path.stem}_{i}.{path.suffix}'
                             .format(**locals()))


def trash(trash_files, trash_dir):
    # get/make trashdir
    if not trash_dir:
        # empty-string and non-existence are treated the same way
        trash_dir = os.environ.get('TRASH_DIR', '')
        if not trash_dir:
            trash_dir = default_trash_dir
    trash_path = Path(trash_dir).expanduser()
    trash_path.mkdir(exist_ok=True, parents=True)

    for trash_file_ in trash_files:
        trash_file = Path(trash_file_)
        if not trash_file.exists():
            print("{}: not found".format(trash_file))
            sys.exit(1)

        # get absolute path
        trash_file = Path(trash_file_).resolve()

        # get a destination which does not already exist
        try:
            trash_dest_candidate = (trash_path /
                                    trash_file.relative_to(Path.home()))
        except ValueError:
            trash_dest_candidate = (trash_path /
                                    '_root' / trash_file.relative_to('/'))

        trash_dest = next(filter(lambda path: not path.exists(),
                                 candidate_names(trash_dest_candidate)))

        trash_dest.parent.mkdir(exist_ok=True, parents=True)
        print('{} -> {}'.format(trash_file, trash_dest))

        try:
            shutil.move(trash_file, trash_dest)
        except Exception as e:
            print(str(e))
            sys.exit(2)


__all__ = ['trash']
